/*
** These programs and files are Copyright October 1998 by Carey
** Bloodworth.
**
** I retain all legal rights and ownership.  However, they are
** provided "as is", without any warranty etc.
**
** I give you permission to use these as needed, without asking
** me for permission, informing me of your use, or any form of payment.
** Feel free to use them if you need to.
**
** You must include a notice in your code and/or documentation that
** the ideas or code came from me.  I've put a lot of work into these
** and it's just fair that my name is included in there somewhere.  You
** don't have to provide a massive explanation, or a copy of my original
** code. But you need to include some notice that I was involved somehow.
** On major projects, I try to give credit where it's due, because it's
** fair.  I just want you to do the same.
**
** You may not put the GNU 'copyleft' on any of my code.  I don't approve
** of it, it's far too restrictive, and I don't want you inflicting
** my code with it.  You can put it on your own code, of course, but if
** you include any of my code, you must clearly indicate that it falls
** under my copyright and is not covered by the GNU stuff.
**
** Carey Bloodworth
** cbloodworth@juno.com
*/

#ifndef __PI_H_
#define __PI_H_ 1

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <math.h>
#include <time.h>
#include <limits.h>
#include <float.h>
#include <ctype.h>

/*IF under Windows95, 98 or NT uncomment following line.
#define IS_WINDOWS
*/

/*IF under Linux or Unix uncomment the following line.
#define IS_UNIX
*/
#define IS_UNIX

#if	defined IS_WINDOWS
	#define QUIT printf("Quitting...\n");
	#define EXIT exit(0);
#elif defined IS_UNIX
	#define QUIT printf("Quitting...\n");
	#define EXIT exit(0);
#else
	#define QUIT printf("Select Quit from the File menu to quit\n");
	#define EXIT exit(0);
#endif

/*
** Set the FFTLimit here. See comments at end of program for details.
#define FFTLimit  1048576
#define FFTLimit  2097152
#define FFTLimit  4194304
#define FFTLimit  8388608
*/

#define FFTLimit  8388608
#ifndef FFTLimit
#error You need to set the FFTLimit for your system.  See program for details.
#endif

/*
** Do you want to use extra memory for another FFT cache?
#define USE_EXTRA_FFT_CACHE 1
#undef USE_EXTRA_FFT_CACHE
*/
#define USE_EXTRA_FFT_CACHE 1

/*
** Stop the user from trying to do this on a 16 bit compiler!
*/
#if SHRT_MAX == INT_MAX
#error Forget it!  A 16 bit compiler would not have enough memory.
#endif

/*
** Should we take advantage of forward transforms always being
** zero padded?  Might as well, since it always works in this
** program, and is slightly faster.  Just made it an option to
** allow you to see how simple the changes are.
#define CUT1
#define CUT2
#define CUT3
#define CUT4
*/
#define ZERO_PAD_CUT
#define CUT4

/* Which iteration to redo for accuracy */
#define REDO_LEN (Len/16)

#ifndef M_SQRT_2
#define M_SQRT_2  0.7071067811865475244008443621
#endif
#define POS (+1.0)
#define NEG (-1.0)

/*
** Half your cache size.  This can be your L1 or L2 cache, which
** ever gives best results.
*/
#define CACHE_SIZE 4096

typedef struct {double r,i;} Cmplx;
typedef short int Short;
typedef long int Long;

/*
** Just a predeclaration.
*/
void Mul(Short *Prod, Short *Num1, Short *Num2, int Len);

/*
** Prototypes
*/
void MakeName(void);
void FatalError(char *fmt,...);
void PrintShort2(char *Str, Short *Num, int Len);
static void PrintDig(int Dig,long printed);
void PrintShort(char *Str, Short *Num, int Len);
int Min(int Num1, int Num2);
void Copy(Short *a, Short *b, int Len);
int DiffWhere(Short *Num1, Short *Num2, int Len);
int IsZero(Short *Num, int Len);
int IsNotZero(Short *Num, int Len);
int CountZeros(Short *Num, int Len);
void DivBy2(Short * Num, int Len);
void DivBy10(Short * Num, int Len);
void MulByFloat(Short *Num, double By, int Len);
void MulBy(Short *Num, int By, int Len);
void MulBy10(Short *Num, int Len);
void CopyProd(Short * a, Short * b, int Len);
Short RippleAdd(Short * limit, Short * sum, Short * a, Short * b, int Len);
Short Add(Short * sum, Short * a, Short * b, int Len);
Short RippleSub(Short * limit, Short * dif, Short * a, Short * b, int Len);
Short Sub(Short * dif, Short * a, Short * b, int Len);
void Negate(Short * num, int Len);
void SlowMul(Short *Prod, Short *Num1, Short *Num2, int Len);
void AddSlowMul(Short *Prod, Short *Num1, Short *Num2, int Len);
void FractalMul(Short *Prod, Short *Num1, Short *Num2, int Len);
void Convolution(double *FFTNum1, double *FFTNum2,int Len);
void FlushFFTCache(void);
void SaveFFTCache(void);
void SwapFFTCache(void);
void RestoreFFTCache(void);
void FFTMul(Short *Prod, Short *Num1, Short *Num2, int Len);
void Mul(Short *Prod, Short *Num1, Short *Num2, int Len);
void DumbMul(Short *Prod, Short *Num1, Short *Num2, int Len);
int EasyMul(Short *Prod, Short *Num1, Short *Num2, int Len);
void HalfMul(Short *Prod, Short *Num1, Short *Num2, int Len);
void SmartMul(Short *Prod, Short *Num1, Short *Num2, int Len);
void Divide(Short *R, Short *Num1, Short *Num2, int Len, int SubLen);
void Sqrt(Short *Root, Short * Num, int Len, int SubLen);
void FourthRoot(Short *Root, Short * Num, int Len, int SubLen);
int Borwein(int Len, int MaxPasses);
int AGM(int Len, int MaxPasses);
void SaveDig(FILE *disk, int Dig,long printed);
void SaveShort(char *Str, Short *Num, int Len);


int Log2(int Num);
void FFTReOrder(double *point, int Len);
void RealFFT(double *ddata,int Len, int Dir);
void FwdRealFFT(double *ddata,int Len);
void RevRealFFT(double *ddata,int Len);
void InitFFT(unsigned long int Len);
void ReOrderShort(Short *Num, int Len);
void LoadFFTNum(Short *Num, double *FFTNum, int Len);

#endif
